import requests
import logging

from collections import deque

logger = logging.getLogger(__name__)


def get_session(cookies=None):
    session = requests.Session()
    if cookies:
        requests.utils.add_dict_to_cookiejar(session.cookies,
                                             cookies)
    return session


def extract_links(page):
    return [a.get("href") for a in page.find_all("a")]


def trim_link(link, domain):
    if domain not in link:
        # External link to another domain
        return None

    link = link.replace("://", "")
    start, end = link.split("/", 1)
    return "/" + end


def create(config):
    root = config.root
    domain = config.domain
    start = config.start_page
    pages = config.pages
    session = get_session(config.cookies)
    headers = config.headers
    proxies = config.proxies
    parser = config.parser
    reader = config.reader

    queue = deque(["{0}/{1}".format(root, start)])
    visited = []

    if config.has_auth():
        auth = config.auth
        req = requests.Request(auth.method, auth.url, data=auth.params)
        resp = session.send(req.prepare())
        if resp.status_code > 400:
            raise RuntimeError("Unable to do authentification")

    while len(queue) > 0:
        link = queue.popleft()
        logger.info("Scrapping: {0}".format(link))
        try:
            data = reader.read_page(session, link, parser, headers, proxies)
        except Exception as ex:
            logger.exception(ex)
            visited.append(link)
            continue

        visited.append(link)

        for page in pages:
            if page.matches_link_pattern(link):
                for mapping in page.mappings:
                    for result in mapping.parse(data):
                        yield result, link

        links = map(lambda x: root + x,
                    filter(lambda x: x is not None,
                           [trim_link(l, domain)
                            for l in extract_links(data)]))

        for link in links:
            if link not in visited:
                queue.append(link)



import os
import re
import yaml

from functools import reduce
from jinja2 import Template


def pattern_to_regex(patt):
    lang = {"%": ".*?",
            "*": "."}
    patt = reduce(lambda a, b: a.replace(b[0], b[1]), lang.items(), patt)
    return re.compile(patt)


class Auth(object):

    """
    Config used to define authentification params
    """

    def __init__(self, cfg):
        self._data = cfg

    @property
    def method(self):
        if "method" in self._data:
            return self._data["method"]
        else:
            return "POST"

    @property
    def url(self):
        return self._data["url"]

    @property
    def params(self):
        return self._data["params"]


class Page(object):

    """
    Represents config subset for concrete page
    """

    def __init__(self, cfg):
        self._data = cfg
        self.matcher = pattern_to_regex(cfg["link_pattern"])
        if "mappings" in cfg:
            self._mappings = [Mapping(m["name"], m["path"])
                              for m in cfg["mappings"]]
        else:
            self._mappings = []

    def matches_link_pattern(self, url):
        return self.matcher.search(url) is not None

    @property
    def mappings(self):
        return self._mappings


class Mapping(object):

    """
    Maps concete text from html element to column
    """

    def __init__(self, key, pattern):
        self.key = key
        self.path = [self.create_node(part)
                     for part in pattern.split("/")]

    def create_node(self, data):
        p = r"(?P<tag>\w+)(?P<q>\[(?P<attr>\w+)=(\"|\')(?P<val>.+?)(\"|\')\])?"
        patt = re.compile(p)

        m = patt.match(data)
        tag = m.group("tag")

        if m.group("q"):
            q = {m.group("attr"): m.group("val")}
        else:
            q = None

        def node(root):
            return root.findAll(tag, q)

        return node

    def parse(self, root):
        results = []
        context = [root]
        for path in self.path:
            for node in context:
                for out in path(node):
                    results.append(out)
        return [(self.key, result.text) for result in results]


class Config(object):

    """
    Class which handles config related stuff
    """

    def __init__(self, f):
        with open(f, "r") as f:
            self._data = self._render(f)

    def __repr__(self):
        return "{0}: {1}".format(self.name, self.desc)

    def _render(self, f):
        tmpl = Template(f.read())

        def env_get():
            return dict(os.environ)

        return yaml.load(tmpl.render(**env_get()))

    @property
    def name(self):
        return self._data["name"]

    @property
    def desc(self):
        return self._data["description"]

    @property
    def root(self):
        return self._data["site_root"]

    @property
    def domain(self):
        root = self.root
        m = re.search(r"https?://(www\.)?(?P<domain>(\w|[.-_])+)", root)
        if m:
            return m.group("domain")
        else:
            return root

    @property
    def start_page(self):
        return self._data["start_page"]

    @property
    def pages(self):
        return [Page(cfg)
                for cfg in self._data["pages"]]

    @property
    def headers(self):
        if "headers" in self._data:
            return self._data["headers"]
        else:
            return None

    @property
    def cookies(self):
        if "cookies" in self._data:
            return self._data["cookies"]
        else:
            return None

    def has_auth(self):
        return "auth" in self._data

    @property
    def auth(self):
        return Auth(self._data["auth"])

    @property
    def parser(self):
        if "parser" in self._data:
            return self._data["parser"]
        else:
            return "html.parser"

    @property
    def proxies(self):
        if "proxies" in self._data:
            return self._data["proxies"]
        else:
            return None

    @property
    def reader(self):
        if "reader" in self._data:
            module = self._data["reader"]
        else:
            module = "grazer.readers.local"
        return __import__(module, fromlist=[""])

    def get_val(self, key):
        return self._data["vars"][key]



import click

from dotenv import load_dotenv, find_dotenv
from grazer.config import Config
from grazer.core import crawler


@click.command()
@click.option("--env", default=find_dotenv())
@click.option("--config")
def main(env, config):
    load_dotenv(env)
    cfg = Config(config)
    for record, link in crawler.create(cfg):
        print(record)

if __name__ == "__main__":
    main()



from bs4 import BeautifulSoup


def read_page(session, url, parser, headers=None, proxies=None):
    raw = session.get(url, headers=headers, proxies=proxies).text
    return BeautifulSoup(raw, parser)



# -*- coding: UTF-8 -*-
from setuptools import setup
from pip.req import parse_requirements

from grazer.version import get_version

install_reqs = list(parse_requirements("requirements.txt", session={}))

setup(name="verata",
      version=get_version(),
      author="Šarūnas Navickas",
      author_email="zaibacu@gmail.com",
      url="https://github.com/CodersOfTheNight/verata",
      packages=["grazer", "grazer.core", "grazer.readers.local"],
      install_requires=[str(ir.req) for ir in install_reqs],
      test_suite="pytest",
      tests_require=["pytest"],
      setup_requires=["pytest-runner"],
      entry_points={"console_scripts": ["verata = grazer.run:main"]}
      )
